package GII::Event::PMove;

=head1 NAME

GII::Event::PMove - Perl class encapsulating gii_pmove_event structures.

=head1 SYNOPSIS

   use GII::Event::PMove;

   $ev = new GII::Event::PMove;

   $x = $ev->x;
   $ev->x($x);

   $y = $ev->y;
   $ev->y($y);

   $z = $ev->z;
   $ev->z($z);

   $wheel = $ev->wheel;
   $ev->wheel($wheel);

   ($x, $y) = $ev->xy;
   $coord->coord($ev->xy);
   $ev->xy($x, $y);
   $ev->xy($coord);

   ($x, $y, $z, $wheel) = $ev->motion;
   $ev->motion($x, $y, $z, $wheel);
   $ev->motion($planar_motion_coord, $z, $wheel);

   $is_relative = $ev->relative;
   $ev->relative($is_relative);

=head1 DESCRIPTION

The GII::Event::PMove subclass provides simple methods to access
the extra members of a LibGII gii_pmove_event structure with cannot
be accessed via the methods in the class GII::Event.

=cut

use strict;
use vars qw($VERSION @ISA);
use GII::Event;

require DynaLoader;
require AutoLoader;

@ISA = qw(AutoLoader DynaLoader GII::Event);

$VERSION = '0.01';

bootstrap GII::Event::PMove $VERSION;

=head1 Methods

The extra methods and differences between GII::Event methods and the
ones provided in the GII::Event::PMove class are as follows:

=head2 new

The GII::Event::PMove constructor takes all the same arguments
as the GII::Event constructor, and treats them the same with the
exception that it does not do any automagical subclassing.  All
calls to this constructor will either fail or allocate/reuse
a gii_pmove_event structure.

Since this event is generated by the environment, this constructor
will be used only very rarely for debugging purposes.

=cut

sub new {
    my($class) = shift();
    my($type) = shift();
    my($this) = {};
    bless $this, $class;
    my($obj) = shift();
    if ($obj) {
        $this->{obj} = $obj;
        $this->set_type($obj, $type) if defined($type);
        $this->set_size($obj, $this->default_size) if defined($type);
        return($this);
    }
    $this->{obj} = $this->_alloc;
    $this->set_size($this->{obj}, $this->default_size);
    $this->set_type($this->{obj}, $type);
    return($this);
}

=head2 relative

Without an argument, returns 1 if the pmove event is of type
evPtrRelative, 0 if the pmove event is of type evPtrAbsolute,
and undef if the type is not a valid pmove event type.  With
an argument, sets the pmove event type to evPtrRelative if 
the argument is true in a boolean context, else sets it to 
evPtrAbsolute, and returns the value which was set.

=cut

sub relative {
    my($this) = shift();
    if (@_ + 0) {
	if (shift()) {
	    $this->set_type($this->{obj}, &evPtrRelative);
	    return &evPtrRelative;
	}
	else {
	    $this->set_type($this->{obj}, &evPtrAbsolute);
            return &evPtrAbsolute;
	}
    }
    return 1 if $this->get_type($this->{obj}) == &evPtrRelative;
    return 0 if $this->get_type($this->{obj}) == &evPtrAbsolute;
    return undef;
}

=head2 x

Without an argument, returns the absolute or relative motion
of the pointer in the horizontal direction.  With an argument, 
sets said value, returning the value which was set.

=cut

sub x {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_x($this->{obj}, shift());
    }
    return $this->get_x($this->{obj});
}

=head2 y

Without an argument, returns the absolute or relative motion
of the pointer in the vertical direction.  With an argument, 
sets said value, returning the value which was set.

=cut

sub y {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_y($this->{obj}, shift());
    }
    return $this->get_y($this->{obj});
}

=head2 z

Without an argument, returns the absolute or relative motion
of the pointer in the plane-orthagonal direction.  With an argument, 
sets said value, returning the value which was set.

=cut

sub z {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_z($this->{obj}, shift());
    }
    return $this->get_z($this->{obj});
}

=head2 wheel

Without an argument, returns the absolute or relative motion
of the mouse wheel.  With an argument, sets said value, returning 
the value which was set.

=cut

sub wheel {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_wheel($this->{obj}, shift());
    }
    return $this->get_wheel($this->{obj});
}

=head2 xy

Without an argument, returns the absolute or relative motion
of the pointer in the horizontal and vertical direction.  
With an argument, sets said values, returning the values which 
were set.

=cut

sub xy {
    my($this) = shift();
    if (@_ + 0) {
        my($arg) = shift();
        return ($this->set_x($this->{obj}, $arg->x),
		$this->set_y($this->{obj}, $arg->y))
            if ref($arg) and UNIVERSAL::isa($arg, 'GGI::Coord');
        return ($this->set_x($this->{obj}, $arg),
		$this->set_y($this->{obj}, shift()));
    }
    return ($this->get_x($this->{obj}),$this->get_y($this->{obj}));
}


=head2 motion

Without an argument, returns the absolute or relative motion
of the pointer in the horizontal, vertical, and plane-orthagonal
directions, and the motion of the mouse wheel.  With an argument, 
sets said values, returning the values which were set.

=cut

sub motion {
    my($this) = shift();
    if (@_ + 0) {
        my($arg) = shift;
        my($x, $y);
        if (ref($arg) and UNIVERSAL::isa($arg, 'GGI::Coord')) {
            $x = $arg->x; 
	    $y = $arg->y;
        }
        else {
	    $x = $arg;
            $y = shift();
        }
        my($arg) = shift();
        return ($this->set_x($this->{obj}, $x),
		$this->set_y($this->{obj}, $y),
                $this->set_z($this->{obj}, $arg->x),
		$this->set_wheel($this->{obj}, $arg->y))
            if ref($arg) and UNIVERSAL::isa($arg, 'GGI::Coord');
        return ($this->set_x($this->{obj}, $x),
		$this->set_y($this->{obj}, $y),
                $this->set_z($this->{obj}, $arg),
		$this->set_wheel($this->{obj}, shift()));
    }
    return ($this->get_x($this->{obj}),
	    $this->get_y($this->{obj}),
	    $this->get_z($this->{obj}),
	    $this->get_wheel($this->{obj}));
}


1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GII::Event(3pm), GII::(3pm), perl(1).

=cut
